//Copyright (c) Microsoft Corporation.  All rights reserved.

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Eas.WebCrawler.Common.Test {
    /// <summary>Sets up a temporary directory for use in testing and copies into the directory any relevant resources.</summary>
    /// <remarks>This class is used for those cases where, as a part of testing, you need to set up a directory, and copy in files which you have embedded as resources into your test project, and you
    /// want to make sure that all this gets cleaned up after every test (plus handle cases where clean-up failed - like debug-stop).</remarks>
    public class TestDirectory : IDisposable{
        /// <summary>The name of the test directory (without it's parent location.</summary>
        /// <remarks>Backing field for the DirectoryName property.</remarks>
        /// <seealso cref="DirectoryName"/>
        private string _DirectoryName;
                
        /// <summary>The full path to the test directory</summary>
        /// <remarks>Backing field for the FullPath property.</remarks>
        /// <seealso cref="FullPath"/>
        private string _FullPath;

        /// <summary>The name of the directory in your project containing resources, or null if there are no resources to copy in.</summary>
        /// <remarks>Backing field for the ResourceDirectoryName property.</remarks>
        /// <seealso cref="ResourceDirectoryName"/>
        private string _ResourceDirectoryName=null;

        private System.Reflection.Assembly _ParentAssembly = null;

        /// <summary>Constructrs a <see cref="TestDirectory"/> from the full path where the directory will be created.</summary>
        /// <param name="directoryName">The full path to where the temporary directory will be created.</param>
        public TestDirectory(string directoryName) : this(directoryName, null, System.Reflection.Assembly.GetCallingAssembly()) {}

        /// <summary>Constructs a <see cref="TestDirectory"/> from the full path where the directory will be created and the name of the folder folding your test resources.</summary>
        /// <param name="directoryName">The full path to where the temporary directory will be created.</param>
        /// <param name="resourceDirectoryName">The name of the directory in which you placed your resources in your visual studio prject file.</param>
        /// <remarks>This constructor assumes that you have placed a collection of test resources into a directory in the test project which was set to build action = Embedded Resources.</remarks>
        public TestDirectory(string directoryName, string resourceDirectoryName) : this(directoryName, resourceDirectoryName, System.Reflection.Assembly.GetCallingAssembly()) { }


        /// <summary>Constructs a <see cref="TestDirectory"/> from the full path where the directory will be created and the name of the folder folding your test resources and the assembly containing test resources.</summary>
        /// <param name="directoryName">The full path to where the temporary directory will be created.</param>
        /// <param name="resourceDirectoryName">The name of the directory in which you placed your resources in your visual studio prject file.</param>
        /// <param name="parentAssembly">The assemcly containing the test resources to be copied.</param>
        /// <remarks>This constructor assumes that you have placed a collection of test resources into a directory in the test project which was set to build action = Embedded Resources.</remarks>
        public TestDirectory(string directoryName, string resourceDirectoryName, System.Reflection.Assembly parentAssembly) {
            _ParentAssembly = parentAssembly;
            _ResourceDirectoryName = resourceDirectoryName;
            DirectoryName = directoryName;
            Cleanup();
            Setup();
        }

        #region IDisposable Members
        /// <summary>Finalizes this instance by deleting the test directory on disk/</summary>
        ~TestDirectory() { Dispose(false); }

        /// <summary>Cleans up this instance by deleting the test directory on disk/</summary>
        public void Dispose() {Dispose(true);}

        /// <summary>Cleans up this instance by deleting the test directory on disk/</summary>
        protected virtual void Dispose(bool isDisposing) {
            if (isDisposing) {
                GC.SuppressFinalize(this);
                Cleanup();
            }
        }

        #endregion

        /// <summary>The name of the test directory (without it's parent location.</summary>
        public string DirectoryName {
            get { return _DirectoryName; }
            set { 
                _DirectoryName = value;
                _FullPath = System.IO.Path.Combine(Path.GetTempPath(), _DirectoryName);
            }
        }

        /// <summary>The name of the directory in your project containing resources.</summary>
        public string ResourceDirectoryName {
            get { return _ResourceDirectoryName; }
        }

        /// <summary>/// <summary>Cleans up this instance by deleting the test directory on disk/</summary></summary>
        public void Cleanup() {
            if (Directory.Exists(_FullPath))
                Directory.Delete(_FullPath, true);
        }

        /// <summary>Cleans out and then recreates the test directory and re-copies in all resources.</summary>
        public void Reinitialize(){
            Cleanup();
            Setup();
        }

        private void Setup() {
            Directory.CreateDirectory(_FullPath);
            CopyResources();
        }

        private void CopyResources() {
            if (_ResourceDirectoryName != null) {
                string prefix = _ParentAssembly.GetName().Name+'.'+_ResourceDirectoryName+'.';
                string[] resources = _ParentAssembly.GetManifestResourceNames();
                foreach (string resource in resources) {
                    if(resource.StartsWith(prefix, StringComparison.Ordinal)){
                        CopyResource(resource);
                    }
                }
            }
        }

        /// <summary>ReCopries a specific resource back into the directory</summary>
        /// <param name="resourceName">The name of the resource to re-copy.</param>
        public void CopyResource(string resourceName) {
            using (System.IO.Stream stream = _ParentAssembly.GetManifestResourceStream(resourceName)) {
                using (System.IO.StreamReader reader = new System.IO.StreamReader(stream)) {
                    string prefix = _ParentAssembly.GetName().Name + '.' + _ResourceDirectoryName + '.';
                    string shortname = resourceName.Substring(prefix.Length);
                    using (System.IO.StreamWriter w = Create(shortname)) {
                        string line = reader.ReadLine();
                        while (line != null) {
                            w.WriteLine(line);
                            line = reader.ReadLine();
                        }
                    }
                }
            }
        }
            
        /// <summary>The full path to the test directory</summary>
        public string FullPath {
            get { return _FullPath; }
        }

        /// <summary>Generate a path to  afile in the test directory.</summary>
        /// <param name="fileName">The name of the file whose path should be generated.</param>
        /// <returns>The full path to the file in the test directory</returns>
        public string CreateFileName(string fileName) {
            return Path.Combine(_FullPath, fileName);
        }

        /// <summary>Create a new, empty text file in the test directory</summary>
        /// <param name="filename">The name (not the full path) of the file to create.</param>
        /// <returns></returns>
        private System.IO.StreamWriter Create(string filename) {
            return System.IO.File.CreateText(Path.Combine(_FullPath, filename));
        }
    }
}
